#!/bin/bash
#
# Title: exifpdf
# Author: TheOuterLinux (https://theouterlinux.gitlab.io)
# License: MIT
# Purpose: Grabs EXIF data from image files and places that
#          information that corresponds to the particular image
#          underneath the image on that image's corresponding PDF.
#
#          Once each PDF is created for that particular image, this
#          script then combines them all into one PDF with the option
#          to encrypt it. And because there is text underneath each
#          image on each page, you now have a PDF with nothing but
#          images, one per page, but contains "hidden," yet 
#          searchable, text on each page.
#
#          For example, if you took a bunch of nature photos and then
#          later used a program, like 'Shotwell', to embed EXIF data
#          into each image, after running this script in those photos'
#          directory, you should then be able to open the resulting PDF
#          and search for a word, such as "mountain," assuming that word
#          was used when adding tags. The PDF viewer would then either
#          list pages to go to with the word "mountain" on them, though
#          hidden from normal view, or just go to the next resulting
#          page.
#
#          Encryption, by default has a key-length of 128; however, it 
#          it is possible to go higher using 'qpdf' but I did not do 
#          this for compatibility's sake as 256-bit forces the PDF 
#          version to 1.6 rather than 1.4. I know that sounds like a
#          weird thing to care about in 2023, but I figured that most
#          users of this script are trying to find a way to get 
#          encrypted photos on older hardware that no one has bothered
#          to make password-protected, container-like software for.
#          Also, 'pdftk' lets you use PROMPT so as to ask for a password
#          while 'qpdf' makes you type it in as part of the script; it
#          should be safer this way.
#
# Last updated: 2023/05/30
# Requirements:
#     - An operating system that can run BASH scripts
#     - coreutils - GNU core utilities
#     - findutils - utilities for finding files--find, xargs
#     - exiftool (libimage-exiftool-perl) : library and program to read
#                                           and write meta information 
#                                           in multimedia files
#     - enscript         : converts text to Postscript, HTML or RTF with
#                          syntax highlighting
#     - img2pdf          : Lossless conversion of raster images to PDF
#     - qpdf             : tools for transforming and inspecting PDF 
#                          files
#     - texlive-latex-recommended : to get the 'ps2pdf' program
#     - pdftk-java : pdftk port to java - a tool for manipulating PDF 
#                    documents
#
CURDIR="$(echo ${PWD##*/})"
OUTPUTPDF="$1"

CLEAN="\e[0m"
GREEN="\e[92m"
BLUE="\e[94m"
YELLOW="\e[93m"
RED="\e[91m"

function HELP(){
    printf ${CLEAN}
    echo ''
    printf ${BLUE}
    echo -n 'Script:  '
    printf ${YELLOW}
    echo 'exifpdf.sh'
    printf ${BLUE}
    echo -n 'Author:  '
    printf ${YELLOW}
    echo 'TheOuterLinux (https://theouterlinux.gitlab.io)'
    printf ${BLUE}
    echo -n 'Purpose: '
    printf ${YELLOW}
    echo 'To convert all of the JPEG images in the current directory to'
    echo '         an encrypted PDF. This script is not recursive; it will not'
    echo '         process images from subdirectories.'
    echo ''
    printf ${RED}
    echo 'Usage:'
    printf ${CLEAN}
    echo ''
    echo '    cd "/path/to/image/directory"'
    echo '    "/path/to/exifpdf.sh" [output.pdf]'
    echo ''
    printf ${GREEN}
    echo 'The "[output.pdf]" part is optional; but without specifying the output,'
    echo 'this script will output the created PDF with the same name as the current'
    echo 'directory. If you do specify the output, make sure to put it in quotes'
    echo 'if it has spaces or special characters in it. The output file can also be'
    echo 'completely somewhere else, such as replacing a PDF in another directory.'
    echo ''
    printf ${CLEAN}
}

function MAIN(){
    if [ "$OUTPUTPDF" = '' ]; then OUTPUTPDF="${CURDIR}.pdf"; fi
    shopt -s nullglob #Suppresses errors regarding "I can't find files with *.ext"
    for f in *.jpg *.jpeg *.JPG *.JPEG *.png *.PNG *.bmp *.BMP *.tiff *.TIFF
    do
        exiftool "$f" > "${f%.*}.exif"
        enscript -p "${f%.*}.ps" --word-wrap "${f%.*}.exif"
        ps2pdf "${f%.*}.ps" "${f%.*}_underlay.pdf"
        rm "${f%.*}.exif"
        rm "${f%.*}.ps"
        img2pdf --output "${f%.*}_image.pdf" "$f"
        qpdf --suppress-recovery --underlay "${f%.*}_underlay.pdf" -- "${f%.*}_image.pdf" "${f%.*}_MergeMe.pdf"
        rm "${f%.*}_underlay.pdf"
        rm "${f%.*}_image.pdf"
    done
    #pdfunite *_MergeMe.pdf "${OUTPUTPDF%.*}_temp.pdf"
    qpdf --empty --pages *_MergeMe.pdf -- "${OUTPUTPDF%.*}_temp.pdf"
    find . -type f -iname "*_MergeMe.pdf" -exec rm "{}" \; #If you have too many image files, this part may look like it doesn't work, but you may just need to refresh the file-manager.
    echo ''
    read -p "Do you want to encrypt the final PDF (y/N)?: " ENCRYPT
    case "$ENCRYPT" in
        'y'|'yes') pdftk "${OUTPUTPDF%.*}_temp.pdf" output "$OUTPUTPDF" user_pw PROMPT && rm "${OUTPUTPDF%.*}_temp.pdf" ;;
        *) mv -f "${OUTPUTPDF%.*}_temp.pdf" "$OUTPUTPDF" ;;
    esac
    echo ''
}

if [ "$1" = '-h' ] || [ "$1" = '--help' ]; then HELP; else MAIN; fi
